package com.yideb.audit.task.dynamic;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.yideb.audit.entity.ScheduleTask;
import com.yideb.audit.service.IScheduleTaskService;
import com.yideb.audit.util.SpringUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.Trigger;
import org.springframework.scheduling.annotation.SchedulingConfigurer;
import org.springframework.scheduling.config.ScheduledTaskRegistrar;
import org.springframework.scheduling.support.CronTrigger;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <b>Application name：</b> yideb-audit-statistic <br>
 * <b>Application describing： </b> <br>
 * <b>Copyright：</b> Copyright &copy; 2019 明医众禾科技（北京）有限责任公司 版权所有。<br>
 * <b>Company：</b> 明医众禾科技（北京）有限责任公司 <br>
 * <b>@Date：</b> 2020/8/5 17:57 <br>
 * <b>@author：</b> <a href="mailto:Lijing@miyzh.com"> Lijing </a> <br>
 */
@Slf4j
@Component
public class DynamicScheduleTask implements SchedulingConfigurer {

	@Resource
	private IScheduleTaskService taskService;

	@Override
	public void configureTasks(ScheduledTaskRegistrar taskRegistrar) {
		List<ScheduleTask> tasks = taskService.findAll();
		log.info("【定时任务启动】 启动任务数：" + tasks.size() + "; time=" + DateUtil.now());

		List<ScheduleTask> list = checkTaskList(tasks);
		if (ObjectUtil.isNotEmpty(list)) {
			for (ScheduleTask task : list) {
				taskRegistrar.addTriggerTask(getRunnable(task), getTrigger(task));
			}
		}

		log.info("【定时任务执行完毕】执行任务数：" + list.size() + "; time=" + DateUtil.now());
	}

	/**
	 * 拼装Runnable
	 *
	 * @param task
	 * @return
	 */
	private Runnable getRunnable(ScheduleTask task) {
		return () -> {
			Object bean = SpringUtil.getBean(task.getClassName());
			try {
				Method method = bean.getClass().getMethod(task.getMethodName(), (Class<?>) null);
				method.invoke(bean);
			} catch (NoSuchMethodException | IllegalAccessException | InvocationTargetException e) {
				e.printStackTrace();
			}
		};
	}

	/**
	 * 获取触发器
	 *
	 * @param task
	 * @return
	 */
	private Trigger getTrigger(ScheduleTask task) {
		return triggerContext -> {
			CronTrigger trigger = new CronTrigger(task.getCron());
			return trigger.nextExecutionTime(triggerContext);
		};
	}

	/**
	 * 过滤有效的任务
	 *
	 * @param list
	 * @return
	 */
	private List<ScheduleTask> checkTaskList(List<ScheduleTask> list) {
		return list.stream().filter(this::checkTaskConfig).collect(Collectors.toList());
	}

	/**
	 * 检查每个配置是否合规
	 *
	 * @param task
	 * @return
	 */
	private boolean checkTaskConfig(ScheduleTask task) {
		boolean flag = false;
		String cron = task.getCron();

		try {
			Class clz = Class.forName(task.getClassName());
			Object bean = SpringUtil.getBean(clz);
			bean.getClass().getMethod(task.getMethodName(), (Class<?>) null);
			if (StrUtil.isBlank(cron)) {
				log.error("没有找到 ClassName:{}, Cron:{}", task.getClassName(), task.getCron());
			}

			flag = true;
		} catch (ClassNotFoundException e) {
			log.error("没有找到 ClassName:{}", task.getClassName(), e);
			e.printStackTrace();
		} catch (NoSuchMethodException e) {
			log.error("没有找到 ClassName:{}, methodName:{}", task.getClassName(), task.getMethodName(), e);
			e.printStackTrace();
		}

		return flag;
	}
}
